<?php
/**
 * Implements hook_rules_action_info
 */
function varnish_tag_invalidate_rules_action_info() {
  $actions = array();
  
  $actions['varnish_add_tag'] = array(
    'parameter' => array(
      'tags' => array(
        'label' => t('Tags'),
        'type' => 'text'
      ),
    ),
    'label' => t('Send tags'),
    'group' => t('Varnish tags'),
    'base' => 'varnish_tag_invalidate_add'
  );
  
  $actions['varnish_tag_invalidate'] = array(
    'parameter' => array(
      'tags' => array(
        'label' => t('Tags'),
        'type' => 'text'
      ),
    ),
    'label' => t('Clear tags'),
    'group' => t('Varnish tags'),
    'base' => 'varnish_tag_invalidate_tags'
  );
  
  return $actions;
}

/**
 * Sends purge 
 * 
 * @param array $tags
 */
function varnish_tag_invalidate_tags($tags) {
  module_load_include('inc', 'purge');
  $tags = array_map('trim', explode("\n", $tags));
  
  $raw_hosts = variable_get('varnish_tag_invalidate_hosts', '');
  $hosts = array_map('trim', explode("\n", $raw_hosts));
  
  $product = varnish_tag_invalidate_cartesian_product(array('tag' => $tags, 'host' => $hosts));
  
  $urls = array_map('varnish_tag_array_map_tag', $product);
  purge_urls($urls);
}

/**
 * Generates cartesian product for hosts and tags
 * 
 * @param array $input
 *  An array with multiple elements to create the cartesian product from 
 *  
 * @return array
 */
function varnish_tag_invalidate_cartesian_product($input) {
  // filter out empty values
  $input = array_filter($input);

  $result = array(array());

  foreach ($input as $key => $values) {
    $append = array();

    foreach($result as $product) {
      foreach($values as $item) {
        $product[$key] = $item;
        $append[] = $product;
      }
    }

    $result = $append;
  }

  return $result;
}

/**
 * Combine the cartesian product of host and tag into a URL to purge
 * 
 * @param array $item
 *  An array containing a key for host and a key for tag
 *  
 * @return string
 */
function varnish_tag_array_map_tag($item) {
  return $item['host'] . '/' . $item['tag'];
}

/**
 * Add appropriate tags to the current page
 * 
 * @param array $tags
 */
function varnish_tag_invalidate_add($tags) {
  $tags = array_map('trim', explode("\n", $tags));
  varnish_tag_invalidate_add_tags($tags);
}